/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.dao;

import org.wso2.registry.LogEntry;
import org.wso2.registry.jdbc.DatabaseConstants;

import java.sql.*;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public class LogsDAO {

    public void addLog(long resourceID, String userName, int action, String actionData, Connection conn)
            throws SQLException {

        String sql = "INSERT INTO LOGS (RID, USER_ID, LOGGED_TIME, ACTION, ACTION_DATA) " +
                "VALUES (?, ?, ?, ?, ?)";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setLong(1, resourceID);
        s.setString(2, userName);
        s.setTimestamp(3, new Timestamp(System.currentTimeMillis()));
        s.setInt(4, action);
        s.setString(5, actionData);

        s.executeUpdate();
    }

    public List getLogs(String resourcePath, int action,
                        String userName, Date from, Date to, Connection conn) throws SQLException {

        VersionedResourceDAO resourceDAO = new VersionedResourceDAO();

        long aid = -1;
        if (resourcePath != null) {
            aid = resourceDAO.getResourceID(resourcePath, conn);
        }

        String sql = "SELECT A.PATH, L.USER_ID, L.LOGGED_TIME, L.ACTION, L.ACTION_DATA FROM " +
                     "ARTIFACTS A, LOGS L WHERE A.AID=L.RID";

        if (aid != -1) {
            sql = sql + " AND L.RID=?";
        }

        if (userName != null) {
            sql = sql + "  AND L.USER_ID=?";
        }

        if (from != null) {
            sql = sql + " AND L.LOGGED_TIME>?";
        }

        if (to != null) {
            sql = sql + " AND L.LOGGED_TIME<?";
        }

        if (action != -1) {
            sql = sql + " AND L.ACTION=?";
        }

        PreparedStatement s = conn.prepareStatement(sql);

        int paramNumber = 1;

        if (aid != -1) {
            s.setLong(paramNumber, aid);
            paramNumber++;
        }

        if (userName != null) {
            s.setString(paramNumber, userName);
            paramNumber++;
        }

        if (from != null) {
            s.setTimestamp(paramNumber, new Timestamp(from.getTime()));
            paramNumber++;
        }

        if (to != null) {
            s.setTimestamp(paramNumber, new Timestamp(to.getTime()));
            paramNumber++;
        }

        if (action != -1) {
            s.setInt(paramNumber, action);
        }

        ResultSet results = s.executeQuery();

        List resultList = new ArrayList();
        while (results.next()) {
            LogEntry logEntry = new LogEntry();
            logEntry.setResourcePath(results.getString(DatabaseConstants.PATH_FIELD));
            logEntry.setUserName(results.getString(DatabaseConstants.USER_ID_FIELD));
            logEntry.setDate(new java.sql.Date(results.getTimestamp(DatabaseConstants.LOGGED_TIME_FIELD).getTime()));
            logEntry.setAction(results.getInt(DatabaseConstants.ACTION_FIELD));
            logEntry.setActionData(results.getString(DatabaseConstants.ACTION_DATA_FIELD));

            resultList.add(logEntry);
        }

        return resultList;
    }
}
