/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.mediatypes.builtin.utils;

import org.apache.ws.commons.schema.XmlSchema;
import org.apache.ws.commons.schema.XmlSchemaCollection;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.NodeList;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import javax.wsdl.Types;
import javax.wsdl.Definition;
import javax.wsdl.Import;
import javax.wsdl.WSDLException;
import javax.wsdl.xml.WSDLReader;
import javax.wsdl.xml.WSDLWriter;
import javax.wsdl.factory.WSDLFactory;
import javax.wsdl.extensions.schema.Schema;
import java.util.*;
import java.io.ByteArrayOutputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;

public class WSDLFileProcessor {

    // remove this when it is not needed
    private int i;

    public static final String IMPORT_TAG = "import";
    public static final String INCLUDE_TAG = "include";

    /**
     * saves a wsdl file including its imports and imported schemas.
     *
     * @param location
     * @throws WSDLException
     */
    public void saveWSDLFileToRegistry(String location)
            throws WSDLException {
        WSDLReader wsdlReader = WSDLFactory.newInstance().newWSDLReader();
        wsdlReader.setFeature("javax.wsdl.importDocuments", true);
        Definition wsdlDefinition = wsdlReader.readWSDL(location);
        Map processedWSDLMap = new HashMap();
        calculateWSDLNamesAndChangeTypes(wsdlDefinition, processedWSDLMap, new HashMap(), new HashSet());
        saveWSDLFileToRegistry(wsdlDefinition, processedWSDLMap, new HashSet());
    }

    /**
     * saves the given wsdl definition file with its imported wsdls and
     * imported and included schemas.
     *
     * @param wsdlDefinition     - wsdl file to save
     * @param processedWSDLMap   - map with original source URI vs new uris for wsdls
     * @param processedScheamMap - map with orignal source URI vs new uris for schemas
     */

    public void calculateWSDLNamesAndChangeTypes(Definition wsdlDefinition,
                                                 Map processedWSDLMap,
                                                 Map processedScheamMap,
                                                 Set visitedWSDLs) {
        // first we have to process the imports and change the
        // schema locations suite for the registry
        Iterator iter = wsdlDefinition.getImports().values().iterator();
        Vector values = null;
        Import wsdlImport = null;
        // add this to visited list to stop recursion
        visitedWSDLs.add(wsdlDefinition.getDocumentBaseURI());
        for (; iter.hasNext();) {
            values = (Vector) iter.next();
            for (Iterator valuesIter = values.iterator(); valuesIter.hasNext();) {
                wsdlImport = (Import) valuesIter.next();
                // process the types recuresiveilt
                Definition innerDefinition = wsdlImport.getDefinition();
                if (!visitedWSDLs.contains(innerDefinition.getDocumentBaseURI())) {
                    // we have not process this wsdl file earlier
                    calculateWSDLNamesAndChangeTypes(
                            innerDefinition, processedWSDLMap, processedScheamMap, visitedWSDLs);
                }
            }
        }

        // change the schema names
        Types types = wsdlDefinition.getTypes();
        if (types != null) {
            List extensibleElements = types.getExtensibilityElements();
            Schema schemaExtension = null;
            Object extensionObject = null;
            XmlSchema xmlSchema = null;
            XmlSchemaCollection xmlSchemaCollection = null;
            SchemaFileProcessor schemaFileProcessor = null;
            Map changedLocationMap = null;
            String basuri = wsdlDefinition.getDocumentBaseURI();
            basuri = basuri.substring(0, basuri.lastIndexOf("/") + 1);
            for (Iterator extensibleElementsIter = extensibleElements.iterator();
                 extensibleElementsIter.hasNext();) {
                extensionObject = extensibleElementsIter.next();
                if (extensionObject instanceof Schema) {
                    // first get the schema object
                    schemaExtension = (Schema) extensionObject;
                    // create an xml schema object to be processed by SchemaFile procesor.
                    xmlSchemaCollection = new XmlSchemaCollection();
                    xmlSchemaCollection.setBaseUri(basuri);
                    xmlSchema = xmlSchemaCollection.read(schemaExtension.getElement());
                    schemaFileProcessor = new SchemaFileProcessor();
                    changedLocationMap = new HashMap();
                    schemaFileProcessor.calculateNewSchemaNames(
                            xmlSchema, processedScheamMap, new HashSet(), true);
                    schemaFileProcessor.saveSchemaFileToRegistry(
                            xmlSchema, processedScheamMap, changedLocationMap, new HashSet(), true);
                    // update the current schema locations with the generated ones.
                    changeLocations(schemaExtension.getElement(), changedLocationMap);
                }
            }
        }

        // after processing the defintions save this to the registry
        // TODO: save this to the registry for the moment save this to the
        // folder and omit any exception occurs.
        String baseUri = wsdlDefinition.getDocumentBaseURI();
        String wsdlFileName = baseUri.substring(baseUri.lastIndexOf("/") + 1);
        String fileNameToSave = wsdlFileName.substring(0, wsdlFileName.indexOf(".")) + ".wsdl";
        while (processedWSDLMap.containsValue(fileNameToSave)) {
            fileNameToSave = wsdlFileName.substring(0, wsdlFileName.indexOf(".")) + ++i + ".wsdl";
        }
        // add this entry to the proccessed wsdl map
        processedWSDLMap.put(baseUri, fileNameToSave);

    }

    public void saveWSDLFileToRegistry(Definition wsdlDefinition,
                                       Map processedWSDLMap,
                                       Set visitedWSDLs) {
        // first we have to process the imports and change the
        // schema locations suite for the registry
        Iterator iter = wsdlDefinition.getImports().values().iterator();
        Vector values = null;
        Import wsdlImport = null;
        // add this to visited list to stop recursion
        visitedWSDLs.add(wsdlDefinition.getDocumentBaseURI());
        for (; iter.hasNext();) {
            values = (Vector) iter.next();
            for (Iterator valuesIter = values.iterator(); valuesIter.hasNext();) {
                wsdlImport = (Import) valuesIter.next();
                // process the types recuresiveilt
                Definition innerDefinition = wsdlImport.getDefinition();
                if (!visitedWSDLs.contains(innerDefinition.getDocumentBaseURI())) {
                    // we have not process this wsdl file earlier
                    saveWSDLFileToRegistry(innerDefinition, processedWSDLMap, visitedWSDLs);
                }
                // set the import location according to the new location
                wsdlImport.setLocationURI((String) processedWSDLMap.get(
                        innerDefinition.getDocumentBaseURI()));
            }
        }

        // after processing the defintions save this to the registry
        // TODO: save this to the registry for the moment save this to the
        // folder and omit any exception occurs.
        String fileNameToSave = (String) processedWSDLMap.get(wsdlDefinition.getDocumentBaseURI());
        try {
            WSDLWriter wsdlWriter = WSDLFactory.newInstance().newWSDLWriter();
            ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
            wsdlWriter.writeWSDL(wsdlDefinition, new FileOutputStream("repository/" + fileNameToSave));
        } catch (WSDLException e) {
            e.printStackTrace();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
    }

    private void changeLocations(Element element, Map changedLocationMap) {
        NodeList nodeList = element.getChildNodes();
        String tagName;
        for (int i = 0; i < nodeList.getLength(); i++) {
            tagName = nodeList.item(i).getLocalName();
            if (IMPORT_TAG.equals(tagName) || INCLUDE_TAG.equals(tagName)) {
                processImport(nodeList.item(i), changedLocationMap);
            }
        }
    }

    private void processImport(Node importNode, Map changedLocationMap) {
        NamedNodeMap nodeMap = importNode.getAttributes();
        Node attribute;
        String attributeValue;
        for (int i = 0; i < nodeMap.getLength(); i++) {
            attribute = nodeMap.item(i);
            if (attribute.getNodeName().equals("schemaLocation")) {
                attributeValue = attribute.getNodeValue();
                if (changedLocationMap.get(attributeValue) != null) {
                    attribute.setNodeValue(
                            (String) changedLocationMap.get(attributeValue));
                }
            }
        }
    }

}
